#include <kvs/StructuredVolumeImporter>
#include <kvs/StructuredVolumeObject>
#include <kvs/Message>
#include <kvs/TransferFunction>
#include <kvs/CellByCellMetropolisSampling>
#include <kvs/PointObject>

#include <kvs/ParticleBasedRenderer>
#include <kvs/Application>
#include <kvs/Screen>


int main( int argc, char** argv )
{
    if( !(argc == 2 || argc == 3) ) { 
      std::cerr << "USAGE (1): PBVR volume_data" ;
      std::cerr << std::endl;
      std::cerr << "USAGE (2): PBVR volume_data transfer_fnction.kvsml" ;
      std::cerr << std::endl;
      exit(1) ;
    }

  // ボリュームデータのImport
  kvs::StructuredVolumeObject* volume =
    new kvs::StructuredVolumeImporter( argv[1] );

  // 伝達関数の設定
  kvs::TransferFunction* transfer_function = 0 ; // 初期化
  if ( argc == 2 ) {
    // rainbow color (default)
    transfer_function =  new kvs::TransferFunction ();
  } else 
  if ( argc == 3 ) {
    // コマンドライン引数で受け取った場合
    transfer_function = new kvs::TransferFunction ( argv[2] );
  }

  // リピートレベルとサブピクセルレベル
  int repeat_level = 9 ;
  int subpixel_level;
  subpixel_level = (int) sqrt ( (double)repeat_level );

  // サンプリングステップとセル幅
  const float sampling_step = 0.5f;
  //  const float object_depth = 1.0f;

  // サンプリング
  kvs::PointObject* object = new kvs::CellByCellMetropolisSampling(
                                                      volume,
                                                      subpixel_level,
                                                      sampling_step,
                                                      *transfer_function );
  // 生成した粒子数の確認
  int num_particles = object->numberOfVertices() ;
  std::cerr << "** Number of Particles: " << num_particles << std::endl;

  // Renderer の生成
  kvs::glsl::ParticleBasedRenderer* renderer = 
      new kvs::glsl::ParticleBasedRenderer(); 
  renderer->setRepetitionLevel ( repeat_level );
  renderer->enableLODControl();


  // windowの初期化
  kvs::Application app( argc, argv );
  // Screenクラスの生成と設定
  kvs::Screen screen( &app );
  screen.setGeometry( 0, 0, 512, 512 );
  // ObjectとRendererの登録
  screen.registerObject( object, renderer );
  screen.setTitle( "KVS Particle Renderer (GLEW)" ) ;
  screen.show();

  return( app.run() );

}
